import { MIDI } from './song';
import { dataBase64 } from '../../static/test_mid_b64';

describe("MIDI file format", () => {
  let m: MIDI;

  beforeAll(() => {
    const arrayBuffer = Uint8Array.from(window.atob(dataBase64), c => c.charCodeAt(0));
    m = new MIDI(arrayBuffer.buffer);
  });

  it("should be able get the data", () => {
    expect(m.numberOfTracks).toEqual(13);
    expect(m.tracks.length).toEqual(13);
  });

  it("should be able get meta data from the track", () => {
    const e0 = m.tracks[0].events[0];
    /* 
    Time Signature
      nn 	numerator of time sig.
      dd 	denominator of time sig.
        2 	quarter
        3 	eighth
        etc
      cc 	number of ticks in metronome click
      bb 	number of 32nd notes to the quarter note
    */
    if (e0.data) {
      // 4/4 time
      expect(e0.type).toEqual('Meta');
      expect(e0.subtype).toEqual('TimeSignature');

      expect(e0.data[0]).toEqual(4);
      expect(e0.data[1]).toEqual(4);

      // number of ticks in metronome click
      expect(e0.data[2]).toEqual(24);
      // number of 32nd notes to the quarter note
      expect(e0.data[3]).toEqual(8);
    }
  });

  it("should be able get meta data from the track 2", () => {
    const e0 = m.tracks[0].events[1];
    /* 
    Key signature
      sf 	sharps/flats
        -7 	7 flats
        0 	key of C
        7 	7 sharps
      mi 	major/minor
        0 	major
        1 	minor
    */
    if (e0.data) {
      expect(e0.type).toEqual('Meta');
      expect(e0.subtype).toEqual('KeySignature');

      // key of C
      expect(e0.data[0]).toEqual(0);
      // major scale
      expect(e0.data[1]).toEqual(0);
    }
  });

  it("should be able get meta data from the track 3", () => {
    const e0 = m.tracks[0].events[2];
    /*
    Set tempo
      tttttt 	microseconds/quarter note
    */
    if (e0.data) {
      expect(e0.type).toEqual('Meta');
      expect(e0.subtype).toEqual('SetTempo');

      expect(e0.data[0]).toEqual(375000);
    }
  });

  it("should have midi events with increasing absolute time", () => {
    const t0 = m.tracks[0];
    let last = 0;
    for (let x = 0; x < t0.events.length; x++) {
      const e = t0.events[x];
      expect(e.abstime >= last).toBeTruthy();
      last = e.abstime;
    }
  });

  // it("...", () => {
  //   console.log(m.tracks[0]);
  // });

});