
import * as Andy from '../andy';
import { Patch } from '../midi/types';

const system = new Andy.SoundSystem();

let waveform: Float32Array;
let spectrum: Uint8Array;
let analyser: AnalyserNode;

function updateWaveForm(time: number) {
  requestAnimationFrame(updateWaveForm);
  analyser.getFloatTimeDomainData(waveform);
  analyser.getByteFrequencyData(spectrum);
  drawOscilloscope(time);
  drawSpectrogram(time);
}

/////////////////////////////////////////
/////////////////////////////////////////
function drawOscilloscope(time: number) {
  // This is not the right place for this, but for demo...
  const scopeCanvas = document.getElementById("oscilloscope") as HTMLCanvasElement;
  scopeCanvas.width = scopeCanvas.clientWidth;
  scopeCanvas.height = scopeCanvas.clientHeight;
  const scopeContext = scopeCanvas.getContext("2d") as CanvasRenderingContext2D;

  scopeContext.clearRect(0, 0, scopeCanvas.width, scopeCanvas.height);
  scopeContext.beginPath();
  for (let i = 0; i < scopeCanvas.width; i++) {
    const x = i; // * (scopeCanvas.width / waveform.length);
    const y = (0.5 + waveform[i] / 2) * scopeCanvas.height;
    if (i == 0) {
      scopeContext.moveTo(x, y);
    } else {
      scopeContext.lineTo(x, y);
    }
  }
  scopeContext.stroke();
}

/////////////////////////////////////////
/////////////////////////////////////////
function drawSpectrogram(time: number) {
  const spectroCanvas = document.getElementById("spectrogram") as HTMLCanvasElement;
  spectroCanvas.width = spectroCanvas.clientWidth;
  spectroCanvas.height = spectroCanvas.clientHeight;
  const canvasCtx = spectroCanvas.getContext("2d") as CanvasRenderingContext2D;

  canvasCtx.clearRect(0, 0, spectroCanvas.width, spectroCanvas.height);

  canvasCtx.fillStyle = "rgb(0, 0, 0)";
  canvasCtx.fillRect(0, 0, spectroCanvas.width, spectroCanvas.height);

  const barWidth = spectroCanvas.width / spectrum.length;
  let barHeight;
  let x = 1;

  for (let i = 0; i < spectrum.length; i++) {
    barHeight = spectrum[i] / 1.2;

    canvasCtx.fillStyle = "rgb(" + (barHeight + 100) + ",50,50)";
    canvasCtx.fillRect(
      x,
      spectroCanvas.height - barHeight / 1.2,
      barWidth,
      barHeight
    );

    x += barWidth;
  }
}

/////////////////////////////////////////
/////////////////////////////////////////
let myPatch: Patch = {
  len: 2,
  vca: { gain: 1 },
  voices: [
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
    {
      vco: { type: 'sine', freqOffset: 0, detune: 0 },
      vca: { gainOffset: 0 },
    },
  ],
  compressor: {
    threshold: -30,
    knee: 40,
    ratio: 12,
    attack: 0,
    release: .25,
  },
  adsr: {
    attack: 0,
    decay: 0,
    sustain: 1,
    release: 1,
  },
  filter: {
    type: 'highpass',
    frequency: 10,
    q: 1.04,
    gain: 0
  }
};

let keyboardActive = true;
const patchArea = document.getElementById('patchText') as HTMLTextAreaElement;
patchArea.addEventListener('focus', () => keyboardActive = false);

patchArea.addEventListener('blur', () => {
  keyboardActive = true;
  try {
    const ptext = patchArea.value;
    myPatch = JSON.parse(ptext);
  } catch (err) {
    console.error(err);
  }
});
const onelineDom = document.getElementById('oneLine') as HTMLInputElement;
onelineDom.addEventListener('click', () => patchArea.value = JSON.stringify(myPatch));

const formatDom = document.getElementById('formatLine') as HTMLInputElement;
formatDom.addEventListener('click', () => patchArea.value = JSON.stringify(myPatch, null, 2));

const loadDom = document.getElementById('load') as HTMLInputElement;
loadDom.addEventListener('click', () => patchArea.value = window.localStorage.getItem('lp') ?? JSON.stringify(myPatch, null, 2));

const saveDom = document.getElementById('save') as HTMLInputElement;
saveDom.addEventListener('click', () => window.localStorage.setItem('lp', patchArea.value));


const keyStatus: any = {};
document.addEventListener('keydown', function (e: KeyboardEvent) {
  if (keyboardActive) {
    const n = Andy.patchPlayer(system, myPatch);
    const speakers = system.masterGainNode();
    switch (e.key) {
      case 'q':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C-2", system.currentTime(), speakers);
        break;
      case '2':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C#2", system.currentTime(), speakers);
        break;
      case 'w':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("D-2", system.currentTime(), speakers);
        break;
      case '3':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("D#2", system.currentTime(), speakers);
        break;
      case 'e':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("E-2", system.currentTime(), speakers);
        break;
      case 'r':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("F-2", system.currentTime(), speakers);
        break;
      case '5':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("F#2", system.currentTime(), speakers);
        break;
      case 't':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("G-2", system.currentTime(), speakers);
        break;
      case '6':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("G#2", system.currentTime(), speakers);
        break;
      case 'y':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("A-2", system.currentTime(), speakers);
        break;
      case '7':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("A#2", system.currentTime(), speakers);
        break;
      case 'u':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("B-2", system.currentTime(), speakers);
        break;
      case 'i':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C-3", system.currentTime(), speakers);
        break;
      ////////////////////////////////////////////////////////////////
      case 'z':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C-4", system.currentTime(), speakers);
        break;
      case 's':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C#4", system.currentTime(), speakers);
        break;
      case 'x':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("D-4", system.currentTime(), speakers);
        break;
      case 'd':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("D#4", system.currentTime(), speakers);
        break;
      case 'c':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("E-4", system.currentTime(), speakers);
        break;
      case 'v':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("F-4", system.currentTime(), speakers);
        break;
      case 'g':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("F#4", system.currentTime(), speakers);
        break;
      case 'b':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("G-4", system.currentTime(), speakers);
        break;
      case 'h':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("G#4", system.currentTime(), speakers);
        break;
      case 'n':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("A-4", system.currentTime(), speakers);
        break;
      case 'j':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("A#4", system.currentTime(), speakers);
        break;
      case 'm':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("B-4", system.currentTime(), speakers);
        break;
      case ',':
        if (!keyStatus[e.key]) keyStatus[e.key] = n("C-5", system.currentTime(), speakers);
        break;
    }
  }
});

document.addEventListener('keyup', function (e: KeyboardEvent) {
  if (keyStatus[e.key]) {
    keyStatus[e.key]();
    delete keyStatus[e.key];
  }
});

patchArea.value = JSON.stringify(myPatch, null, 2);

/////////////////////////////////////////
/////////////////////////////////////////

analyser = system.getAnalyser();
// analyser.fftSize = 2048;
analyser.fftSize = 4096;
waveform = new Float32Array(analyser.frequencyBinCount);
spectrum = new Uint8Array(analyser.frequencyBinCount);

updateWaveForm(0);
drawOscilloscope(0);
drawSpectrogram(0);

const volumeDom = document.getElementById("volume") as HTMLInputElement;
volumeDom.value = system.volume + '';
volumeDom.addEventListener('input', (e: Event) => {
  system.adjustVolume(parseFloat((e.target as HTMLInputElement).value));
});

